/*****************************************************************************
 * gyachi_lib.c
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Copyright (C) 2003 Erica Andrews (PhrozenSmoke ['at'] yahoo.com)
 * Released under the terms of the GPL.
 * *NO WARRANTY*
 *
 * common routines for gyachi, and assorted utilities
 *****************************************************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libintl.h>
#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "gyachi_lib.h"
#include "gy_config.h"


/*        For historical notes, protocol numbers       */
/*  0x09000000 - old, YMSG-9                           */
/*  0x0A000000 - old YMSG10                            */ 
/*  0x0a00c800 - latest  'official' Linux Yahoo client */ 
/*  0x000B0000 - current: YMSG-11, Messenger 5.6       */
/*  0x000C0000 - current: YMSG-12, Messenger 6 Beta    */
/*  0x000D0000 - current: YMSG-13, Messenger 7.5 Beta  */
/*  0x000F0000 - current: YMSG-15, Messenger 8.0       */

YMSG_PROTOCOL YMSG_PROTOCOLS[] = {
  { YMSG_VER_11, "YMSG-11 / Messenger 5.6" } ,
  { YMSG_VER_13, "YMSG-13 / Messenger 7.5 Beta" } ,
  { YMSG_VER_15, "YMSG-15 / Messenger 8" },
  { 0,          0 }
};

char *_(char *some) 
{
/*
#ifdef USE_GTK2
	static gchar *my_locale_string=NULL;

	if (my_locale_string != NULL) {g_free(my_locale_string);}
	my_locale_string=g_locale_to_utf8(gettext(some),-1, NULL, NULL, NULL );
	return my_locale_string;
#endif */
	return gettext(some);
}

gchar *_utf(char *some) { /* converts from the assumed locale to UTF */
	static gchar *mygconvedstr=NULL;

	if (mygconvedstr) {g_free(mygconvedstr); mygconvedstr=NULL;}
	if (g_utf8_validate ( some, -1, NULL))  { /* already in utf */
		mygconvedstr=g_strdup(some);
		return mygconvedstr;
	}
	mygconvedstr=g_convert(some, -1,"UTF-8","ISO8859-1",NULL,NULL,NULL);
	if (! mygconvedstr) {mygconvedstr=g_strdup(some);}
	return mygconvedstr;
}

gchar *_b2loc(char *some) {  /* converts back from UTF to locale string */
	static gchar *my_locale_string=NULL;

	if (my_locale_string) {g_free(my_locale_string); my_locale_string=NULL;}
	if (! g_utf8_validate ( some, -1, NULL))  {
		my_locale_string=g_strdup(some);
		return my_locale_string;
	}
	my_locale_string=g_locale_from_utf8(some, -1, NULL, NULL,NULL);
	if (! my_locale_string) {my_locale_string=g_strdup(some);}
	return my_locale_string;
}

int yprotocol_name_to_protocol(const char *proto)
{
	YMSG_PROTOCOL *y_proto;

	for (y_proto = YMSG_PROTOCOLS; y_proto->protocol; y_proto++) {
		if (strcmp(y_proto->proto_name, proto) == 0) return(y_proto->protocol);
	}
	return 0;
}

const char *yprotocol_to_name(int proto)
{
	YMSG_PROTOCOL *y_proto;

	for (y_proto = YMSG_PROTOCOLS; y_proto->protocol; y_proto++) {
		if (y_proto->protocol == proto) return(y_proto->proto_name);
	}
	return 0;
}


/* common code very commonly used to set a tooltip */
void gyachi_set_tooltip(GtkWidget *somewid, char *somechar) {
#ifdef HAS_GTK_WIDGET_TOOLTIP
	gtk_widget_set_tooltip_text(somewid, somechar);
#else
	GtkTooltips *TIP_GROUP;

	if (!somewid) {return;}
	TIP_GROUP = GTK_TOOLTIPS(g_object_get_data(G_OBJECT(somewid), "tooltip_group"));
	if ( TIP_GROUP == NULL ) {
		TIP_GROUP = gtk_tooltips_new();
		g_object_set_data(G_OBJECT(somewid), "tooltip_group", TIP_GROUP);
	}

	if (!somechar) {
		gtk_tooltips_disable(TIP_GROUP);
		return;
	}

	gtk_tooltips_set_tip(TIP_GROUP, somewid, somechar, somechar);
	gtk_tooltips_enable(TIP_GROUP);
#endif
}


GtkWidget *gyachi_combobox_new(int editable)
{
	GtkWidget *gy_combo_box;

	if (editable) {
		gy_combo_box = gtk_combo_box_entry_new_text();
	}
	else {
		gy_combo_box = gtk_combo_box_new_text();
	}
	gtk_widget_show (gy_combo_box);
	return gy_combo_box;
}

void gyachi_combobox_load(GtkWidget *gy_combo_box, char *item_array[])
{
	char  **item_pointer;

	for (item_pointer = item_array; *item_pointer; item_pointer++) {
		gtk_combo_box_append_text(GTK_COMBO_BOX(gy_combo_box), *item_pointer);
	}
}

void gyachi_combobox_load_list(GtkWidget *gy_combo_box, GList *item_list)
{
	GList *this_item;
	char  *item;

	for (this_item=item_list; this_item; this_item=g_list_next(this_item)) {
		item=(char *)this_item->data;
		gtk_combo_box_append_text(GTK_COMBO_BOX(gy_combo_box), item);
	}
}

void gyachi_combobox_prepend_item(GtkWidget *gy_combo_box, const char *item) {
	gtk_combo_box_prepend_text(GTK_COMBO_BOX(gy_combo_box), item);
}

void gyachi_combobox_append_item(GtkWidget *gy_combo_box, const char *item) {
	gtk_combo_box_append_text(GTK_COMBO_BOX(gy_combo_box), item);
}

void gyachi_combobox_select_itemno(GtkWidget *gy_combo_box, int selected_item_no)
{
	gtk_combo_box_set_active(GTK_COMBO_BOX(gy_combo_box), selected_item_no);
}

/* returns 1 (TRUE) if item is found/selected, 0 (FALSE) otherwise */
int gyachi_combobox_select_item(GtkWidget *gy_combo_box, char *item)
{
	GtkTreeModel *model;
	GtkTreeIter   combobox_iter;
	gboolean      valid;
	gchar        *text = NULL;
	int           selected_item_no;
	int           found = FALSE;

	g_object_get(GTK_COMBO_BOX(gy_combo_box), "model", &model, NULL);
	valid = gtk_tree_model_get_iter_first(model, &combobox_iter);
	selected_item_no = 0;
	while (valid) {
		gtk_tree_model_get(model, &combobox_iter, 0, &text, -1);
		if (!strcmp(text, item)) {
			found=TRUE;
			g_free(text);
			break;
		}
		g_free(text);
		selected_item_no++;
		valid = gtk_tree_model_iter_next(model, &combobox_iter);
	}
	if (found) {
		gyachi_combobox_select_itemno(gy_combo_box, selected_item_no);
	}

	return(found);
}

const char *gyachi_combobox_get_selected_item(GtkWidget *gy_combo_box)
{
	static char *text = NULL;

	if (text) {
		free(text);
		text = NULL;
	}

	text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(gy_combo_box));
	return(text);
}

GtkWidget *lookup_widget(GtkWidget *widget, const gchar *widget_name)
{
	GtkWidget *parent, *found_widget;

	for (;;) {
		if (GTK_IS_MENU (widget)) {
			parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
		}
		else {
			parent = widget->parent;
		}
		if (!parent) {
			parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");
		}
		if (parent == NULL) {
			break;
		}
		widget = parent;
	}

	found_widget = (GtkWidget*) g_object_get_data (G_OBJECT (widget),
                                                 widget_name);
	if (!found_widget) {
		g_warning ("Widget not found: %s", widget_name);
	}
	return found_widget;
}

YMSG_ROOM *find_room(char *room_name) {
	GList     *this_item;
	YMSG_ROOM *room;

	for (this_item=room_list; this_item; this_item=g_list_next(this_item)) {
		room=(YMSG_ROOM*)this_item->data;
		if (!strcmp(room_name, room->room_name)) {
			return(room);
		}
	}
	return NULL;
}

void add_room(char *room_name, char *room_id, int fav) {
	YMSG_ROOM *room;

	room = find_room(room_name);
	if (room) {
		if (room_id) {
			free(room->room_id);
			room->room_id = strdup(room_id);
		}
		if (fav >= 0) {
			room->fav = fav;
		}

		return;
	}

	room = malloc(sizeof(YMSG_ROOM));
	room->room_name = strdup(room_name);
	room->room_id   = strdup(room_id?room_id:"0");
	room->fav       = (fav>=0)?fav:0;
	room_list = g_list_append(room_list, room);
}
